@@ -1,14 +1,27 @@
-/*
-    SPDX-FileCopyrightText: 2016 Martin Gräßlin <mgraesslin@kde.org>
+/********************************************************************
+Copyright 2016  Martin Gräßlin <mgraesslin@kde.org>
 
-    SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
-*/
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) version 3, or any
+later version accepted by the membership of KDE e.V. (or its
+successor approved by the membership of KDE e.V.), which shall
+act as a proxy defined in Section 6 of version 3 of the license.
+
+This library is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
+
+You should have received a copy of the GNU Lesser General Public
+License along with this library.  If not, see <http://www.gnu.org/licenses/>.
+*********************************************************************/
 // Qt
-#include <QSignalSpy>
-#include <QTest>
+#include <QtTest>
 // client
-#include "../../src/client/compositor.h"
 #include "../../src/client/connection_thread.h"
+#include "../../src/client/compositor.h"
 #include "../../src/client/event_queue.h"
 #include "../../src/client/pointer.h"
 #include "../../src/client/pointerconstraints.h"
@@ -16,8 +29,8 @@
 #include "../../src/client/seat.h"
 #include "../../src/client/surface.h"
 // server
-#include "../../src/server/compositor_interface.h"
 #include "../../src/server/display.h"
+#include "../../src/server/compositor_interface.h"
 #include "../../src/server/pointerconstraints_interface.h"
 #include "../../src/server/seat_interface.h"
 #include "../../src/server/surface_interface.h"
@@ -103,19 +116,16 @@ void TestPointerConstraints::init()
     registry.setup();
     QVERIFY(interfacesAnnouncedSpy.wait());
 
-    auto compositorInterface = registry.interface(Registry::Interface::Compositor);
-    m_compositor = registry.createCompositor(compositorInterface.name, compositorInterface.version, this);
+    m_compositor = registry.createCompositor(registry.interface(Registry::Interface::Compositor).name, registry.interface(Registry::Interface::Compositor).version, this);
     QVERIFY(m_compositor);
     QVERIFY(m_compositor->isValid());
 
     m_pointerConstraints = registry.createPointerConstraints(registry.interface(Registry::Interface::PointerConstraintsUnstableV1).name,
-                                                             registry.interface(Registry::Interface::PointerConstraintsUnstableV1).version,
-                                                             this);
+                                                             registry.interface(Registry::Interface::PointerConstraintsUnstableV1).version, this);
     QVERIFY(m_pointerConstraints);
     QVERIFY(m_pointerConstraints->isValid());
 
-    auto seatInterface = registry.interface(Registry::Interface::Seat);
-    m_seat = registry.createSeat(seatInterface.name, seatInterface.version, this);
+    m_seat = registry.createSeat(registry.interface(Registry::Interface::Seat).name, registry.interface(Registry::Interface::Seat).version, this);
     QVERIFY(m_seat);
     QVERIFY(m_seat->isValid());
     QSignalSpy pointerChangedSpy(m_seat, &Seat::hasPointerChanged);
@@ -127,10 +137,10 @@ void TestPointerConstraints::init()
 
 void TestPointerConstraints::cleanup()
 {
-#define CLEANUP(variable)                                                                                                                                      \
-    if (variable) {                                                                                                                                            \
-        delete variable;                                                                                                                                       \
-        variable = nullptr;                                                                                                                                    \
+#define CLEANUP(variable) \
+    if (variable) { \
+        delete variable; \
+        variable = nullptr; \
     }
     CLEANUP(m_compositor)
     CLEANUP(m_pointerConstraints)
@@ -149,7 +159,7 @@ void TestPointerConstraints::cleanup()
     }
 
     CLEANUP(m_compositorInterface)
-    CLEANUP(m_seatInterface)
+    CLEANUP(m_seatInterface);
     CLEANUP(m_pointerConstraintsInterface)
     CLEANUP(m_display)
 #undef CLEANUP
@@ -176,7 +186,7 @@ void TestPointerConstraints::testLockPointer()
     QVERIFY(surface->isValid());
     QVERIFY(surfaceCreatedSpy.wait());
 
-    auto serverSurface = surfaceCreatedSpy.first().first().value<SurfaceInterface *>();
+    auto serverSurface = surfaceCreatedSpy.first().first().value<SurfaceInterface*>();
     QVERIFY(serverSurface);
     QVERIFY(serverSurface->lockedPointer().isNull());
     QVERIFY(serverSurface->confinedPointer().isNull());
@@ -290,7 +300,7 @@ void TestPointerConstraints::testConfinePointer()
     QVERIFY(surface->isValid());
     QVERIFY(surfaceCreatedSpy.wait());
 
-    auto serverSurface = surfaceCreatedSpy.first().first().value<SurfaceInterface *>();
+    auto serverSurface = surfaceCreatedSpy.first().first().value<SurfaceInterface*>();
     QVERIFY(serverSurface);
     QVERIFY(serverSurface->lockedPointer().isNull());
     QVERIFY(serverSurface->confinedPointer().isNull());
@@ -364,7 +374,7 @@ void TestPointerConstraints::testConfinePointer()
 
 enum class Constraint {
     Lock,
-    Confine,
+    Confine
 };
 
 Q_DECLARE_METATYPE(Constraint)
