@@ -1,5 +1,6 @@
 /********************************************************************
 Copyright 2014  Martin Gräßlin <mgraesslin@kde.org>
+Copyright 2018  David Edmundson <davidedmundson@kde.org>
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -25,8 +26,10 @@ License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 #include "outputmanagement_interface.h"
 #include "outputdevice_interface.h"
 #include "idle_interface.h"
+#include "idleinhibit_interface_p.h"
+#include "remote_access_interface.h"
 #include "fakeinput_interface.h"
-#include "logging_p.h"
+#include "logging.h"
 #include "output_interface.h"
 #include "plasmashell_interface.h"
 #include "plasmawindowmanagement_interface.h"
@@ -44,7 +47,14 @@ License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 #include "subcompositor_interface.h"
 #include "textinput_interface_p.h"
 #include "xdgshell_v5_interface_p.h"
+#include "xdgforeign_interface.h"
 #include "xdgshell_v6_interface_p.h"
+#include "xdgshell_stable_interface_p.h"
+#include "appmenu_interface.h"
+#include "server_decoration_palette_interface.h"
+#include "plasmavirtualdesktop_interface.h"
+#include "xdgoutput_interface.h"
+#include "xdgdecoration_interface.h"
 
 #include <QCoreApplication>
 #include <QDebug>
@@ -288,6 +298,13 @@ QtSurfaceExtensionInterface *Display::createQtSurfaceExtension(QObject *parent)
     return s;
 }
 
+RemoteAccessManagerInterface *Display::createRemoteAccessManager(QObject *parent)
+{
+    auto i = new RemoteAccessManagerInterface(this, parent);
+    connect(this, &Display::aboutToTerminate, i, [this, i] { delete i; });
+    return i;
+}
+
 IdleInterface *Display::createIdle(QObject *parent)
 {
     auto i = new IdleInterface(this, parent);
@@ -371,6 +388,9 @@ XdgShellInterface *Display::createXdgShell(const XdgShellInterfaceVersion &versi
     case XdgShellInterfaceVersion::UnstableV6:
         x = new XdgShellV6Interface(this, parent);
         break;
+    case XdgShellInterfaceVersion::Stable:
+        x = new XdgShellStableInterface(this, parent);
+        break;
     }
     connect(this, &Display::aboutToTerminate, x, [x] { delete x; });
     return x;
@@ -412,6 +432,61 @@ PointerConstraintsInterface *Display::createPointerConstraints(const PointerCons
     return p;
 }
 
+XdgForeignInterface *Display::createXdgForeignInterface(QObject *parent)
+{
+    XdgForeignInterface *foreign = new XdgForeignInterface(this, parent);
+    connect(this, &Display::aboutToTerminate, foreign, [this,foreign] { delete foreign; });
+    return foreign;
+}
+
+IdleInhibitManagerInterface *Display::createIdleInhibitManager(const IdleInhibitManagerInterfaceVersion &version, QObject *parent)
+{
+    IdleInhibitManagerInterface *i = nullptr;
+    switch (version) {
+    case IdleInhibitManagerInterfaceVersion::UnstableV1:
+        i = new IdleInhibitManagerUnstableV1Interface(this, parent);
+        break;
+    }
+    connect(this, &Display::aboutToTerminate, i, [this,i] { delete i; });
+    return i;
+}
+
+AppMenuManagerInterface *Display::createAppMenuManagerInterface(QObject *parent)
+{
+    auto b = new AppMenuManagerInterface(this, parent);
+    connect(this, &Display::aboutToTerminate, b, [this, b] { delete b; });
+    return b;
+}
+
+ServerSideDecorationPaletteManagerInterface *Display::createServerSideDecorationPaletteManager(QObject *parent)
+{
+    auto b = new ServerSideDecorationPaletteManagerInterface(this, parent);
+    connect(this, &Display::aboutToTerminate, b, [this, b] { delete b; });
+    return b;
+}
+
+
+PlasmaVirtualDesktopManagementInterface *Display::createPlasmaVirtualDesktopManagement(QObject *parent)
+{
+    auto b = new PlasmaVirtualDesktopManagementInterface(this, parent);
+    connect(this, &Display::aboutToTerminate, b, [this, b] { delete b; });
+    return b;
+}
+
+XdgOutputManagerInterface *Display::createXdgOutputManager(QObject *parent)
+{
+    auto b = new XdgOutputManagerInterface(this, parent);
+    connect(this, &Display::aboutToTerminate, b, [this, b] { delete b; });
+    return b;
+}
+
+XdgDecorationManagerInterface *Display::createXdgDecorationManager(XdgShellInterface *shellInterface, QObject *parent)
+{
+    auto d = new XdgDecorationManagerInterface(this, shellInterface, parent);
+    connect(this, &Display::aboutToTerminate, d, [d] { delete d; });
+    return d;
+}
+
 void Display::createShm()
 {
     Q_ASSERT(d->display);
