@@ -1,5 +1,6 @@
 /********************************************************************
 Copyright 2014  Martin Gräßlin <mgraesslin@kde.org>
+Copyright 2018  David Edmundson <davidedmundson@kde.org>
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -54,6 +55,7 @@ class DataDeviceManagerInterface;
 class DpmsManagerInterface;
 class IdleInterface;
 enum class IdleInhibitManagerInterfaceVersion;
+class RemoteAccessManagerInterface;
 class IdleInhibitManagerInterface;
 class FakeInputInterface;
 class OutputInterface;
@@ -85,6 +87,9 @@ class PointerConstraintsInterface;
 class XdgForeignInterface;
 class AppMenuManagerInterface;
 class ServerSideDecorationPaletteManagerInterface;
+class PlasmaVirtualDesktopManagementInterface;
+class XdgOutputManagerInterface;
+class XdgDecorationManagerInterface;
 
 /**
  * @brief Class holding the Wayland server display loop.
@@ -176,6 +181,7 @@ public:
     PlasmaWindowManagementInterface *createPlasmaWindowManagement(QObject *parent = nullptr);
     QtSurfaceExtensionInterface *createQtSurfaceExtension(QObject *parent = nullptr);
     IdleInterface *createIdle(QObject *parent = nullptr);
+    RemoteAccessManagerInterface *createRemoteAccessManager(QObject *parent = nullptr);
     FakeInputInterface *createFakeInput(QObject *parent = nullptr);
     ShadowManagerInterface *createShadowManager(QObject *parent = nullptr);
     BlurManagerInterface *createBlurManager(QObject *parent = nullptr);
@@ -256,6 +262,31 @@ public:
      **/
     ServerSideDecorationPaletteManagerInterface *createServerSideDecorationPaletteManager(QObject *parent = nullptr);
 
+    /**
+     * Creates the XdgOutputManagerInterface
+     *
+     * @return the created manager
+     * @since 5.47
+     */
+    XdgOutputManagerInterface *createXdgOutputManager(QObject *parent = nullptr);
+
+
+    /**
+     * Creates the PlasmaVirtualDesktopManagementInterface in interface @p version.
+     *
+     * @returns The created manager object
+     * @since 5.52
+     **/
+    PlasmaVirtualDesktopManagementInterface *createPlasmaVirtualDesktopManagement(QObject *parent = nullptr);
+
+    /**
+     * Creates the XdgDecorationManagerInterface
+     * @arg shellInterface A created XdgShellInterface based on XDG_WM_BASE
+     *
+     * @return the created manager
+     * @since 5.54
+     */
+    XdgDecorationManagerInterface *createXdgDecorationManager(XdgShellInterface *shellInterface, QObject *parent = nullptr);
 
     /**
      * Gets the ClientConnection for the given @p client.
